<?php

namespace Asadi\Guardian\Http\Controllers;

use App\Http\Controllers\AdminController as BaseController;
use Form;
use Asadi\Guardian\Http\Requests\GuardianRequest;
use Asadi\Guardian\Interfaces\GuardianRepositoryInterface;
use Asadi\Guardian\Models\Guardian;

/**
 * Admin web controller class.
 */
class GuardianAdminController extends BaseController
{
    // use GuardianWorkflow;
    /**
     * Initialize guardian controller.
     *
     * @param type GuardianRepositoryInterface $guardian
     *
     * @return type
     */
    public function __construct(GuardianRepositoryInterface $guardian)
    {
        $this->repository = $guardian;
        parent::__construct();
    }

    /**
     * Display a list of guardian.
     *
     * @return Response
     */
    public function index(GuardianRequest $request)
    {
        if ($request->wantsJson()) {
            return $this->getJson($request);
        }
        $this   ->theme->prependTitle(trans('guardian::guardian.names').' :: ');
        return $this->theme->of('guardian::admin.guardian.index')->render();
    }

    /**
     * Display a list of guardian.
     *
     * @return Response
     */
    public function getJson(GuardianRequest $request)
    {
        $pageLimit = $request->input('pageLimit');

        $guardians  = $this->repository
                ->pushCriteria(app('Litepie\Repository\Criteria\RequestCriteria'))
                ->setPresenter('\\Asadi\\Guardian\\Repositories\\Presenter\\GuardianListPresenter')
                ->scopeQuery(function($query){
                    return $query->orderBy('id','DESC');
                })->paginate($pageLimit);
        $guardians['recordsTotal']    = $guardians['meta']['pagination']['total'];
        $guardians['recordsFiltered'] = $guardians['meta']['pagination']['total'];
        $guardians['request']         = $request->all();
        return response()->json($guardians, 200);

    }

    /**
     * Display guardian.
     *
     * @param Request $request
     * @param Model   $guardian
     *
     * @return Response
     */
    public function show(GuardianRequest $request, Guardian $guardian)
    {
        if (!$guardian->exists) {
            return response()->view('guardian::admin.guardian.new', compact('guardian'));
        }

        Form::populate($guardian);
        return response()->view('guardian::admin.guardian.show', compact('guardian'));
    }

    /**
     * Show the form for creating a new guardian.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function create(GuardianRequest $request)
    {

        $guardian = $this->repository->newInstance([]);

        Form::populate($guardian);

        return response()->view('guardian::admin.guardian.create', compact('guardian'));

    }

    /**
     * Create new guardian.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(GuardianRequest $request)
    {
        try {
            $attributes             = $request->all();
            $attributes['user_id']  = user_id('admin.web');
            $guardian          = $this->repository->create($attributes);

            return response()->json([
                'message'  => trans('messages.success.updated', ['Module' => trans('guardian::guardian.name')]),
                'code'     => 204,
                'redirect' => trans_url('/admin/guardian/guardian/'.$guardian->getRouteKey())
            ], 201);


        } catch (Exception $e) {
            return response()->json([
                'message'  => $e->getMessage(),
                'code'     => 400,
            ], 400);
        }
    }

    /**
     * Show guardian for editing.
     *
     * @param Request $request
     * @param Model   $guardian
     *
     * @return Response
     */
    public function edit(GuardianRequest $request, Guardian $guardian)
    {
        Form::populate($guardian);
        return  response()->view('guardian::admin.guardian.edit', compact('guardian'));
    }

    /**
     * Update the guardian.
     *
     * @param Request $request
     * @param Model   $guardian
     *
     * @return Response
     */
    public function update(GuardianRequest $request, Guardian $guardian)
    {
        try {

            $attributes = $request->all();

            $guardian->update($attributes);

            return response()->json([
                'message'  => trans('messages.success.updated', ['Module' => trans('guardian::guardian.name')]),
                'code'     => 204,
                'redirect' => trans_url('/admin/guardian/guardian/'.$guardian->getRouteKey())
            ], 201);

        } catch (Exception $e) {

            return response()->json([
                'message'  => $e->getMessage(),
                'code'     => 400,
                'redirect' => trans_url('/admin/guardian/guardian/'.$guardian->getRouteKey()),
            ], 400);

        }
    }

    /**
     * Remove the guardian.
     *
     * @param Model   $guardian
     *
     * @return Response
     */
    public function destroy(GuardianRequest $request, Guardian $guardian)
    {

        try {

            $t = $guardian->delete();

            return response()->json([
                'message'  => trans('messages.success.deleted', ['Module' => trans('guardian::guardian.name')]),
                'code'     => 202,
                'redirect' => trans_url('/admin/guardian/guardian/0'),
            ], 202);

        } catch (Exception $e) {

            return response()->json([
                'message'  => $e->getMessage(),
                'code'     => 400,
                'redirect' => trans_url('/admin/guardian/guardian/'.$guardian->getRouteKey()),
            ], 400);
        }
    }

}
